#  SEQUENCE FREQUENCY ANALYSIS - DNA most_frequent_seq - Consensus Reference Builder

##############################################################
# MIT License

# Copyright (c) [2020] [ZACHARY L. DWIGHT]

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
##############################################################
#
#
# Use for creating consensus or reference sequence from array of seqs

#EXAMPLE
#require 'most_frequent_seq'
#seqs = ['tgactgactgatcgatcgatcgnaaa','tgactgactgatcgatcgatcgaagg','tgactgactgatcgatcgatcgaggg','tgactggctgatcgatcgatcgaccc','tgactg-ctgatcgctcgatcgaccg','agactgactgancgctcgatcgccct']
#puts CONSENSUS::Sequences.calcConsensus(seqs)
# >> TGACTGACTGATCGATCGATCGACCG


module CONSENSUS
class Sequences
	def self.calcConsensus(seqs)
		seql = seqs[0].length - 1
		seqr = ""

		#loop through each base index
		for i in 0..seql

			#establish array for counting bases at i position
			bases = [0,0,0,0,0,0,0] # A,G,C,T,N,-,U
			
			#for each seq, count the type of base at each index
			seqs.each { |n| 
				n = n.upcase
				if(n[i]=='A')
					bases[0] = bases[0]+1
				end
				if(n[i]=='G')
					bases[1] = bases[1]+1
				end
				if(n[i]=='C')
					bases[2] = bases[2]+1
				end
				if(n[i]=='T')
					bases[3] = bases[3]+1
				end
				if(n[i]=='N')
					bases[4] = bases[4]+1
				end
				if(n[i]=='-')
					bases[5] = bases[5]+1
				end
				if(n[i]=='U')
					bases[6] = bases[6]+1
				end
				
				}
			
			#find the max frequency , most frequent base at this index
			cb = bases.each_with_index.max[1]

			#convert back to string
			if(cb==0)
				base = 'A'
			end
			if(cb==1)
				base = 'G'
			end
			if(cb==2)
				base = 'C'
			end
			if(cb==3)
				base = 'T'
			end
			if(cb==4)
				base = 'N'
			end
			if(cb==5)
				base = '-'
			end
			if(cb==6)
				base = 'U'
			end
			
			#concat base to our consensus sequence
			seqr.concat(base.to_s)
				 
			
			end
		return seqr
end
end
end
